@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.web.test.test_webclient -*-
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -28,6 +28,13 @@ class PartialDownloadError(error.Error):
 
 
 class HTTPPageGetter(http.HTTPClient):
+    """
+    Gets a resource via HTTP, then quits.
+
+    Typically used with L{HTTPClientFactory}.  Note that this class does not, by
+    itself, do anything with the response.  If you want to download a resource
+    into a file, use L{HTTPPageDownloader} instead.
+    """
 
     quietLoss = 0
     followRedirect = True
@@ -38,7 +45,13 @@ class HTTPPageGetter(http.HTTPClient):
     def connectionMade(self):
         method = getattr(self.factory, 'method', 'GET')
         self.sendCommand(method, self.factory.path)
-        self.sendHeader('Host', self.factory.headers.get("host", self.factory.host))
+        if self.factory.scheme == 'http' and self.factory.port != 80:
+            host = '%s:%s' % (self.factory.host, self.factory.port)
+        elif self.factory.scheme == 'https' and self.factory.port != 443:
+            host = '%s:%s' % (self.factory.host, self.factory.port)
+        else:
+            host = self.factory.host
+        self.sendHeader('Host', self.factory.headers.get("host", host))
         self.sendHeader('User-Agent', self.factory.agent)
         data = getattr(self.factory, 'postdata', None)
         if data is not None:
@@ -62,8 +75,18 @@ class HTTPPageGetter(http.HTTPClient):
             self.transport.write(data)
 
     def handleHeader(self, key, value):
+        """
+        Called every time a header is received. Stores the header information
+        as key-value pairs in the C{headers} attribute.
+
+        @type key: C{str}
+        @param key: An HTTP header field name.
+
+        @type value: C{str}
+        @param value: An HTTP header field value.
+        """
         key = key.lower()
-        l = self.headers[key] = self.headers.get(key, [])
+        l = self.headers.setdefault(key, [])
         l.append(value)
 
     def handleStatus(self, version, status, message):
@@ -127,7 +150,8 @@ class HTTPPageGetter(http.HTTPClient):
     def handleStatus_302(self):
         if self.afterFoundGet:
             self.handleStatus_303()
-        self.handleStatus_301()
+        else:
+            self.handleStatus_301()
 
 
     def handleStatus_303(self):
@@ -147,7 +171,7 @@ class HTTPPageGetter(http.HTTPClient):
                 failure.Failure(
                     error.Error(
                         self.status, self.message, response)))
-        if self.factory.method.upper() == 'HEAD':
+        if self.factory.method == 'HEAD':
             # Callback with empty string, since there is never a response
             # body for HEAD requests.
             self.factory.page('')
@@ -188,7 +212,7 @@ class HTTPPageDownloader(HTTPPageGetter):
             self.transmittingPage = 0
             self.factory.noPage(
                 failure.Failure(
-                    PartialDownloadError(None)))
+                    PartialDownloadError(self.status)))
         if self.transmittingPage:
             self.factory.pageEnd()
             self.transmittingPage = 0
@@ -343,7 +367,7 @@ class HTTPDownloader(HTTPClientFactory):
                  method='GET', postdata=None, headers=None,
                  agent="Twisted client", supportPartial=0,
                  timeout=0, cookies=None, followRedirect=1,
-                 redirectLimit=20):
+                 redirectLimit=20, afterFoundGet=False):
         self.requestedPartial = 0
         if isinstance(fileOrName, types.StringTypes):
             self.fileName = fileOrName
@@ -360,7 +384,8 @@ class HTTPDownloader(HTTPClientFactory):
         HTTPClientFactory.__init__(
             self, url, method=method, postdata=postdata, headers=headers,
             agent=agent, timeout=timeout, cookies=cookies,
-            followRedirect=followRedirect, redirectLimit=redirectLimit)
+            followRedirect=followRedirect, redirectLimit=redirectLimit,
+            afterFoundGet=afterFoundGet)
 
 
     def gotHeaders(self, headers):
@@ -513,7 +538,7 @@ def getPage(url, contextFactory=None, *args, **kwargs):
     Download a page. Return a deferred, which will callback with a
     page (as a string) or errback with a description of the error.
 
-    See HTTPClientFactory to see what extra args can be passed.
+    See L{HTTPClientFactory} to see what extra arguments can be passed.
     """
     return _makeGetterFactory(
         url,
@@ -547,20 +572,118 @@ from twisted.web.error import SchemeNotSupported
 from twisted.web._newclient import ResponseDone, Request, HTTP11ClientProtocol
 from twisted.web._newclient import Response
 
+try:
+    from twisted.internet.ssl import ClientContextFactory
+except ImportError:
+    class WebClientContextFactory(object):
+        """
+        A web context factory which doesn't work because the necessary SSL
+        support is missing.
+        """
+        def getContext(self, hostname, port):
+            raise NotImplementedError("SSL support unavailable")
+else:
+    class WebClientContextFactory(ClientContextFactory):
+        """
+        A web context factory which ignores the hostname and port and does no
+        certificate verification.
+        """
+        def getContext(self, hostname, port):
+            return ClientContextFactory.getContext(self)
+
+
+
+class _WebToNormalContextFactory(object):
+    """
+    Adapt a web context factory to a normal context factory.
+
+    @ivar _webContext: A web context factory which accepts a hostname and port
+        number to its C{getContext} method.
+
+    @ivar _hostname: The hostname which will be passed to
+        C{_webContext.getContext}.
+
+    @ivar _port: The port number which will be passed to
+        C{_webContext.getContext}.
+    """
+    def __init__(self, webContext, hostname, port):
+        self._webContext = webContext
+        self._hostname = hostname
+        self._port = port
+
+
+    def getContext(self):
+        """
+        Called the wrapped web context factory's C{getContext} method with a
+        hostname and port number and return the resulting context object.
+        """
+        return self._webContext.getContext(self._hostname, self._port)
+
+
+
 class Agent(object):
     """
-    L{Agent} is a very basic HTTP client.  It supports I{HTTP} scheme URIs.  It
-    does not support persistent connections.
+    L{Agent} is a very basic HTTP client.  It supports I{HTTP} and I{HTTPS}
+    scheme URIs (but performs no certificate checking by default).  It does not
+    support persistent connections.
 
-    @ivar _reactor: The L{IReactorTCP} implementation which will be used to set
-        up connections over which to issue requests.
+    @ivar _reactor: The L{IReactorTCP} and L{IReactorSSL} implementation which
+        will be used to set up connections over which to issue requests.
+
+    @ivar _contextFactory: A web context factory which will be used to create
+        SSL context objects for any SSL connections the agent needs to make.
 
     @since: 9.0
     """
     _protocol = HTTP11ClientProtocol
 
-    def __init__(self, reactor):
+    def __init__(self, reactor, contextFactory=WebClientContextFactory()):
         self._reactor = reactor
+        self._contextFactory = contextFactory
+
+
+    def _wrapContextFactory(self, host, port):
+        """
+        Create and return a normal context factory wrapped around
+        C{self._contextFactory} in such a way that C{self._contextFactory} will
+        have the host and port information passed to it.
+
+        @param host: A C{str} giving the hostname which will be connected to in
+            order to issue a request.
+
+        @param port: An C{int} giving the port number the connection will be on.
+
+        @return: A context factory suitable to be passed to C{reactor.connectSSL}.
+        """
+        return _WebToNormalContextFactory(self._contextFactory, host, port)
+
+
+    def _connect(self, scheme, host, port):
+        """
+        Connect to the given host and port, using a transport selected based on
+        scheme.
+
+        @param scheme: A string like C{'http'} or C{'https'} (the only two
+            supported values) to use to determine how to establish the
+            connection.
+
+        @param host: A C{str} giving the hostname which will be connected to in
+            order to issue a request.
+
+        @param port: An C{int} giving the port number the connection will be on.
+
+        @return: A L{Deferred} which fires with a connected instance of
+            C{self._protocol}.
+        """
+        cc = ClientCreator(self._reactor, self._protocol)
+        if scheme == 'http':
+            d = cc.connectTCP(host, port)
+        elif scheme == 'https':
+            d = cc.connectSSL(host, port, self._wrapContextFactory(host, port))
+        else:
+            d = defer.fail(SchemeNotSupported(
+                    "Unsupported scheme: %r" % (scheme,)))
+        return d
 
 
     def request(self, method, uri, headers=None, bodyProducer=None):
@@ -589,11 +712,7 @@ class Agent(object):
         @rtype: L{Deferred}
         """
         scheme, host, port, path = _parse(uri)
-        if scheme != 'http':
-            return defer.fail(SchemeNotSupported(
-                    "Unsupported scheme: %r" % (scheme,)))
-        cc = ClientCreator(self._reactor, self._protocol)
-        d = cc.connectTCP(host, port)
+        d = self._connect(scheme, host, port)
         if headers is None:
             headers = Headers()
         if not headers.hasHeader('host'):
@@ -613,7 +732,7 @@ class Agent(object):
         Compute the string to use for the value of the I{Host} header, based on
         the given scheme, host name, and port number.
         """
-        if port == 80:
+        if (scheme, port) in (('http', 80), ('https', 443)):
             return host
         return '%s:%d' % (host, port)
 
