@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.web.test.test_proxy -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -33,7 +33,11 @@ from twisted.web.http import HTTPClient, Request, HTTPChannel
 class ProxyClient(HTTPClient):
     """
     Used by ProxyClientFactory to implement a simple web proxy.
+
+    @ivar _finished: A flag which indicates whether or not the original request
+        has been finished yet.
     """
+    _finished = False
 
     def __init__(self, command, rest, version, headers, data, father):
         self.father = father
@@ -42,6 +46,7 @@ class ProxyClient(HTTPClient):
         if "proxy-connection" in headers:
             del headers["proxy-connection"]
         headers["connection"] = "close"
+        headers.pop('keep-alive', None)
         self.headers = headers
         self.data = data
 
@@ -55,28 +60,33 @@ class ProxyClient(HTTPClient):
 
 
     def handleStatus(self, version, code, message):
-        if message:
-            # Add a whitespace to message, this allows empty messages
-            # transparently
-            message = " %s" % (message,)
-        self.father.transport.write("%s %s%s\r\n" % (version, code, message))
+        self.father.setResponseCode(int(code), message)
 
 
     def handleHeader(self, key, value):
-        self.father.transport.write("%s: %s\r\n" % (key, value))
-
-
-    def handleEndHeaders(self):
-        self.father.transport.write("\r\n")
+        # t.web.server.Request sets default values for these headers in its
+        # 'process' method. When these headers are received from the remote
+        # server, they ought to override the defaults, rather than append to
+        # them.
+        if key.lower() in ['server', 'date', 'content-type']:
+            self.father.responseHeaders.setRawHeaders(key, [value])
+        else:
+            self.father.responseHeaders.addRawHeader(key, value)
 
 
     def handleResponsePart(self, buffer):
-        self.father.transport.write(buffer)
+        self.father.write(buffer)
 
 
     def handleResponseEnd(self):
-        self.transport.loseConnection()
-        self.father.channel.transport.loseConnection()
+        """
+        Finish the original request, indicating that the response has been
+        completely written to it, and disconnect the outgoing transport.
+        """
+        if not self._finished:
+            self._finished = True
+            self.father.finish()
+            self.transport.loseConnection()
 
 
 
@@ -103,11 +113,14 @@ class ProxyClientFactory(ClientFactory):
 
 
     def clientConnectionFailed(self, connector, reason):
-        self.father.transport.write("HTTP/1.0 501 Gateway error\r\n")
-        self.father.transport.write("Content-Type: text/html\r\n")
-        self.father.transport.write("\r\n")
-        self.father.transport.write('''<H1>Could not connect</H1>''')
-        self.father.transport.loseConnection()
+        """
+        Report a connection failure in a response to the incoming request as
+        an error.
+        """
+        self.father.setResponseCode(501, "Gateway error")
+        self.father.responseHeaders.addRawHeader("Content-Type", "text/html")
+        self.father.write("<H1>Could not connect</H1>")
+        self.father.finish()
 
 
 
@@ -189,6 +202,11 @@ class ReverseProxyRequest(Request):
 
 
     def process(self):
+        """
+        Handle this request by connecting to the proxied server and forwarding
+        it there, then forwarding the response back as the response to this
+        request.
+        """
         self.received_headers['host'] = self.factory.host
         clientFactory = self.proxyClientFactoryClass(
             self.method, self.uri, self.clientproto, self.getAllHeaders(),
@@ -257,7 +275,8 @@ class ReverseProxyResource(Resource):
         C{path} at the end.
         """
         return ReverseProxyResource(
-            self.host, self.port, self.path + '/' + urlquote(path, safe=""))
+            self.host, self.port, self.path + '/' + urlquote(path, safe=""),
+            self.reactor)
 
 
     def render(self, request):
@@ -267,9 +286,10 @@ class ReverseProxyResource(Resource):
         # RFC 2616 tells us that we can omit the port if it's the default port,
         # but we have to provide it otherwise
         if self.port == 80:
-            request.received_headers['host'] = self.host
+            host = self.host
         else:
-            request.received_headers['host'] = "%s:%d" % (self.host, self.port)
+            host = "%s:%d" % (self.host, self.port)
+        request.received_headers['host'] = host
         request.content.seek(0, 0)
         qs = urlparse.urlparse(request.uri)[4]
         if qs:
