@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -881,24 +881,6 @@ Hello,
 
 
 class QueryArgumentsTestCase(unittest.TestCase):
-    def testUnquote(self):
-        try:
-            from twisted.protocols import _c_urlarg
-        except ImportError:
-            raise unittest.SkipTest("_c_urlarg module is not available")
-        # work exactly like urllib.unquote, including stupid things
-        # % followed by a non-hexdigit in the middle and in the end
-        self.failUnlessEqual(urllib.unquote("%notreally%n"),
-            _c_urlarg.unquote("%notreally%n"))
-        # % followed by hexdigit, followed by non-hexdigit
-        self.failUnlessEqual(urllib.unquote("%1quite%1"),
-            _c_urlarg.unquote("%1quite%1"))
-        # unquoted text, followed by some quoted chars, ends in a trailing %
-        self.failUnlessEqual(urllib.unquote("blah%21%40%23blah%"),
-            _c_urlarg.unquote("blah%21%40%23blah%"))
-        # Empty string
-        self.failUnlessEqual(urllib.unquote(""), _c_urlarg.unquote(""))
-
     def testParseqs(self):
         self.failUnlessEqual(cgi.parse_qs("a=b&d=c;+=f"),
             http.parse_qs("a=b&d=c;+=f"))
@@ -972,13 +954,6 @@ class QueryArgumentsTestCase(unittest.TestCase):
         self.assertRaises(TypeError, http.urlparse, u'http://example.org/path')
 
 
-    def testEscchar(self):
-        try:
-            from twisted.protocols import _c_urlarg
-        except ImportError:
-            raise unittest.SkipTest("_c_urlarg module is not available")
-        self.failUnlessEqual("!@#+b",
-            _c_urlarg.unquote("+21+40+23+b", "+"))
 
 class ClientDriver(http.HTTPClient):
     def handleStatus(self, version, status, message):
@@ -1149,13 +1124,51 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
     def test_setHost(self):
         """
         L{http.Request.setHost} sets the value of the host request header.
+        The port should not be added because it is the default.
         """
         req = http.Request(DummyChannel(), None)
-        req.setHost("example.com", 443)
-        self.assertEqual(
+        req.setHost("example.com", 80)
+        self.assertEquals(
             req.requestHeaders.getRawHeaders("host"), ["example.com"])
 
 
+    def test_setHostSSL(self):
+        """
+        L{http.Request.setHost} sets the value of the host request header.
+        The port should not be added because it is the default.
+        """
+        d = DummyChannel()
+        d.transport = DummyChannel.SSL()
+        req = http.Request(d, None)
+        req.setHost("example.com", 443)
+        self.assertEquals(
+            req.requestHeaders.getRawHeaders("host"), ["example.com"])
+
+
+    def test_setHostNonDefaultPort(self):
+        """
+        L{http.Request.setHost} sets the value of the host request header.
+        The port should be added because it is not the default.
+        """
+        req = http.Request(DummyChannel(), None)
+        req.setHost("example.com", 81)
+        self.assertEquals(
+            req.requestHeaders.getRawHeaders("host"), ["example.com:81"])
+
+
+    def test_setHostSSLNonDefaultPort(self):
+        """
+        L{http.Request.setHost} sets the value of the host request header.
+        The port should be added because it is not the default.
+        """
+        d = DummyChannel()
+        d.transport = DummyChannel.SSL()
+        req = http.Request(d, None)
+        req.setHost("example.com", 81)
+        self.assertEquals(
+            req.requestHeaders.getRawHeaders("host"), ["example.com:81"])
+
+
     def test_setHeader(self):
         """
         L{http.Request.setHeader} sets the value of the given response header.
@@ -1407,6 +1420,22 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
         return finished
 
 
+    def test_writeAfterFinish(self):
+        """
+        Calling L{Request.write} after L{Request.finish} has been called results
+        in a L{RuntimeError} being raised.
+        """
+        request = http.Request(DummyChannel(), False)
+        finished = request.notifyFinish()
+        # Force the request to have a non-None content attribute.  This is
+        # probably a bug in Request.
+        request.gotLength(1)
+        request.write('foobar')
+        request.finish()
+        self.assertRaises(RuntimeError, request.write, 'foobar')
+        return finished
+
+
     def test_finishAfterConnectionLost(self):
         """
         Calling L{Request.finish} after L{Request.connectionLost} has been
@@ -1417,3 +1446,115 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
         req = http.Request(channel, False)
         req.connectionLost(Failure(ConnectionLost("The end.")))
         self.assertRaises(RuntimeError, req.finish)
+
+
+
+class MultilineHeadersTestCase(unittest.TestCase):
+    """
+    Tests to exercise handling of multiline headers by L{HTTPClient}.  RFCs 1945
+    (HTTP 1.0) and 2616 (HTTP 1.1) state that HTTP message header fields can
+    span multiple lines if each extra line is preceded by at least one space or
+    horizontal tab.
+    """
+    def setUp(self):
+        """
+        Initialize variables used to verify that the header-processing functions
+        are getting called.
+        """
+        self.handleHeaderCalled = False
+        self.handleEndHeadersCalled = False
+
+    # Dictionary of sample complete HTTP header key/value pairs, including
+    # multiline headers.
+    expectedHeaders = {'Content-Length': '10',
+                       'X-Multiline' : 'line-0\tline-1',
+                       'X-Multiline2' : 'line-2 line-3'}
+
+    def ourHandleHeader(self, key, val):
+        """
+        Dummy implementation of L{HTTPClient.handleHeader}.
+        """
+        self.handleHeaderCalled = True
+        self.assertEquals(val, self.expectedHeaders[key])
+
+
+    def ourHandleEndHeaders(self):
+        """
+        Dummy implementation of L{HTTPClient.handleEndHeaders}.
+        """
+        self.handleEndHeadersCalled = True
+
+
+    def test_extractHeader(self):
+        """
+        A header isn't processed by L{HTTPClient.extractHeader} until it is
+        confirmed in L{HTTPClient.lineReceived} that the header has been
+        received completely.
+        """
+        c = ClientDriver()
+        c.handleHeader = self.ourHandleHeader
+        c.handleEndHeaders = self.ourHandleEndHeaders
+
+        c.lineReceived('HTTP/1.0 201')
+        c.lineReceived('Content-Length: 10')
+        self.assertIdentical(c.length, None)
+        self.assertFalse(self.handleHeaderCalled)
+        self.assertFalse(self.handleEndHeadersCalled)
+
+        # Signal end of headers.
+        c.lineReceived('')
+        self.assertTrue(self.handleHeaderCalled)
+        self.assertTrue(self.handleEndHeadersCalled)
+
+        self.assertEquals(c.length, 10)
+
+
+    def test_noHeaders(self):
+        """
+        An HTTP request with no headers will not cause any calls to
+        L{handleHeader} but will cause L{handleEndHeaders} to be called on
+        L{HTTPClient} subclasses.
+        """
+        c = ClientDriver()
+        c.handleHeader = self.ourHandleHeader
+        c.handleEndHeaders = self.ourHandleEndHeaders
+        c.lineReceived('HTTP/1.0 201')
+
+        # Signal end of headers.
+        c.lineReceived('')
+        self.assertFalse(self.handleHeaderCalled)
+        self.assertTrue(self.handleEndHeadersCalled)
+
+        self.assertEquals(c.version, 'HTTP/1.0')
+        self.assertEquals(c.status, '201')
+
+
+    def test_multilineHeaders(self):
+        """
+        L{HTTPClient} parses multiline headers by buffering header lines until
+        an empty line or a line that does not start with whitespace hits
+        lineReceived, confirming that the header has been received completely.
+        """
+        c = ClientDriver()
+        c.handleHeader = self.ourHandleHeader
+        c.handleEndHeaders = self.ourHandleEndHeaders
+
+        c.lineReceived('HTTP/1.0 201')
+        c.lineReceived('X-Multiline: line-0')
+        self.assertFalse(self.handleHeaderCalled)
+        # Start continuing line with a tab.
+        c.lineReceived('\tline-1')
+        c.lineReceived('X-Multiline2: line-2')
+        # The previous header must be complete, so now it can be processed.
+        self.assertTrue(self.handleHeaderCalled)
+        # Start continuing line with a space.
+        c.lineReceived(' line-3')
+        c.lineReceived('Content-Length: 10')
+
+        # Signal end of headers.
+        c.lineReceived('')
+        self.assertTrue(self.handleEndHeadersCalled)
+
+        self.assertEquals(c.version, 'HTTP/1.0')
+        self.assertEquals(c.status, '201')
+        self.assertEquals(c.length, 10)
