@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -8,13 +8,15 @@ Tests for various parts of L{twisted.web}.
 from cStringIO import StringIO
 
 from zope.interface import implements
+from zope.interface.verify import verifyObject
 
 from twisted.trial import unittest
+from twisted.internet import reactor
 from twisted.internet.address import IPv4Address
 from twisted.internet.defer import Deferred
 from twisted.web import server, resource, util
-from twisted.internet import defer, interfaces, error, task
-from twisted.web import http, http_headers
+from twisted.internet import defer, interfaces, task
+from twisted.web import iweb, http, http_headers, error
 from twisted.python import log
 
 
@@ -96,6 +98,27 @@ class DummyRequest:
         self.session = self.protoSession
         return self.session
 
+
+    def render(self, resource):
+        """
+        Render the given resource as a response to this request.
+
+        This implementation only handles a few of the most common behaviors of
+        resources.  It can handle a render method that returns a string or
+        C{NOT_DONE_YET}.  It doesn't know anything about the semantics of
+        request methods (eg HEAD) nor how to set any particular headers.
+        Basically, it's largely broken, but sufficient for some tests at least.
+        It should B{not} be expanded to do all the same stuff L{Request} does.
+        Instead, L{DummyRequest} should be phased out and L{Request} (or some
+        other real code factored in a different way) used.
+        """
+        result = resource.render(self)
+        if result is server.NOT_DONE_YET:
+            return
+        self.write(result)
+        self.finish()
+
+
     def write(self, data):
         self.written.append(data)
 
@@ -138,13 +161,14 @@ class DummyRequest:
         self.args[name] = [value]
 
 
-    def setResponseCode(self, code):
+    def setResponseCode(self, code, message=None):
         """
         Set the HTTP status response code, but takes care that this is called
         before any data is written.
         """
         assert not self.written, "Response code cannot be set after data has been written: %s." % "@@@@".join(self.written)
         self.responseCode = code
+        self.responseMessage = message
 
 
     def setLastModified(self, when):
@@ -183,9 +207,11 @@ class SimpleResource(resource.Resource):
 class DummyChannel:
     class TCP:
         port = 80
+        disconnected = False
 
         def __init__(self):
             self.written = StringIO()
+            self.producers = []
 
         def getPeer(self):
             return IPv4Address("TCP", '192.168.1.1', 12344)
@@ -200,6 +226,13 @@ class DummyChannel:
         def getHost(self):
             return IPv4Address("TCP", '10.0.0.1', self.port)
 
+        def registerProducer(self, producer, streaming):
+            self.producers.append((producer, streaming))
+
+        def loseConnection(self):
+            self.disconnected = True
+
+
     class SSL(TCP):
         implements(interfaces.ISSLTransport)
 
@@ -214,7 +247,6 @@ class DummyChannel:
 
 
 
-
 class SiteTest(unittest.TestCase):
     def test_simplestSite(self):
         """
@@ -232,74 +264,132 @@ class SiteTest(unittest.TestCase):
 
 
 class SessionTest(unittest.TestCase):
-
+    """
+    Tests for L{server.Session}.
+    """
     def setUp(self):
         """
-        Set up a session using a simulated scheduler. Creates a
-        C{times} attribute which specifies the return values of the
-        session's C{_getTime} method.
+        Create a site with one active session using a deterministic, easily
+        controlled clock.
         """
-        clock = self.clock = task.Clock()
-        times = self.times = []
-
-        class MockSession(server.Session):
-            """
-            A mock L{server.Session} object which fakes out scheduling
-            with the C{clock} attribute and fakes out the current time
-            to be the elements of L{SessionTest}'s C{times} attribute.
-            """
-            def loopFactory(self, *a, **kw):
-                """
-                Create a L{task.LoopingCall} which uses
-                L{SessionTest}'s C{clock} attribute.
-                """
-                call = task.LoopingCall(*a, **kw)
-                call.clock = clock
-                return call
-
-            def _getTime(self):
-                return times.pop(0)
-
-        self.site = server.Site(SimpleResource())
-        self.site.sessionFactory = MockSession
+        self.clock = task.Clock()
+        self.uid = 'unique'
+        self.site = server.Site(resource.Resource())
+        self.session = server.Session(self.site, self.uid, self.clock)
+        self.site.sessions[self.uid] = self.session
 
 
-    def test_basicExpiration(self):
+    def test_defaultReactor(self):
         """
-        Test session expiration: setup a session, and simulate an expiration
-        time.
+        If not value is passed to L{server.Session.__init__}, the global
+        reactor is used.
         """
-        self.times.extend([0, server.Session.sessionTimeout + 1])
-        session = self.site.makeSession()
-        hasExpired = [False]
-        def cbExpire():
-            hasExpired[0] = True
-        session.notifyOnExpire(cbExpire)
-        self.clock.advance(server.Site.sessionCheckTime - 1)
-        # Looping call should not have been executed
-        self.failIf(hasExpired[0])
+        session = server.Session(server.Site(resource.Resource()), '123')
+        self.assertIdentical(session._reactor, reactor)
 
+
+    def test_startCheckingExpiration(self):
+        """
+        L{server.Session.startCheckingExpiration} causes the session to expire
+        after L{server.Session.sessionTimeout} seconds without activity.
+        """
+        self.session.startCheckingExpiration()
+
+        # Advance to almost the timeout - nothing should happen.
+        self.clock.advance(self.session.sessionTimeout - 1)
+        self.assertIn(self.uid, self.site.sessions)
+
+        # Advance to the timeout, the session should expire.
         self.clock.advance(1)
+        self.assertNotIn(self.uid, self.site.sessions)
 
-        self.failUnless(hasExpired[0])
+        # There should be no calls left over, either.
+        self.assertFalse(self.clock.calls)
 
 
-    def test_delayedCallCleanup(self):
+    def test_expire(self):
         """
-        Checking to make sure Sessions do not leave extra DelayedCalls.
+        L{server.Session.expire} expires the session.
         """
-        self.times.extend([0, 100])
+        self.session.expire()
+        # It should be gone from the session dictionary.
+        self.assertNotIn(self.uid, self.site.sessions)
+        # And there should be no pending delayed calls.
+        self.assertFalse(self.clock.calls)
 
-        session = self.site.makeSession()
-        loop = session.checkExpiredLoop
-        session.touch()
-        self.failUnless(loop.running)
 
-        session.expire()
+    def test_expireWhileChecking(self):
+        """
+        L{server.Session.expire} expires the session even if the timeout call
+        isn't due yet.
+        """
+        self.session.startCheckingExpiration()
+        self.test_expire()
 
-        self.failIf(self.clock.calls)
-        self.failIf(loop.running)
 
+    def test_notifyOnExpire(self):
+        """
+        A function registered with L{server.Session.notifyOnExpire} is called
+        when the session expires.
+        """
+        callbackRan = [False]
+        def expired():
+            callbackRan[0] = True
+        self.session.notifyOnExpire(expired)
+        self.session.expire()
+        self.assertTrue(callbackRan[0])
+
+
+    def test_touch(self):
+        """
+        L{server.Session.touch} updates L{server.Session.lastModified} and
+        delays session timeout.
+        """
+        # Make sure it works before startCheckingExpiration
+        self.clock.advance(3)
+        self.session.touch()
+        self.assertEqual(self.session.lastModified, 3)
+
+        # And after startCheckingExpiration
+        self.session.startCheckingExpiration()
+        self.clock.advance(self.session.sessionTimeout - 1)
+        self.session.touch()
+        self.clock.advance(self.session.sessionTimeout - 1)
+        self.assertIn(self.uid, self.site.sessions)
+
+        # It should have advanced it by just sessionTimeout, no more.
+        self.clock.advance(1)
+        self.assertNotIn(self.uid, self.site.sessions)
+
+
+    def test_startCheckingExpirationParameterDeprecated(self):
+        """
+        L{server.Session.startCheckingExpiration} emits a deprecation warning
+        if it is invoked with a parameter.
+        """
+        self.session.startCheckingExpiration(123)
+        warnings = self.flushWarnings([
+                self.test_startCheckingExpirationParameterDeprecated])
+        self.assertEqual(len(warnings), 1)
+        self.assertEqual(warnings[0]['category'], DeprecationWarning)
+        self.assertEqual(
+            warnings[0]['message'],
+            "The lifetime parameter to startCheckingExpiration is deprecated "
+            "since Twisted 9.0.  See Session.sessionTimeout instead.")
+
+
+    def test_checkExpiredDeprecated(self):
+        """
+        L{server.Session.checkExpired} is deprecated.
+        """
+        self.session.checkExpired()
+        warnings = self.flushWarnings([self.test_checkExpiredDeprecated])
+        self.assertEqual(warnings[0]['category'], DeprecationWarning)
+        self.assertEqual(
+            warnings[0]['message'],
+            "Session.checkExpired is deprecated since Twisted 9.0; sessions "
+            "check themselves now, you don't need to.")
+        self.assertEqual(len(warnings), 1)
 
 
 # Conditional requests:
@@ -327,7 +417,9 @@ def httpCode(whole):
     return int(l1.split()[1])
 
 class ConditionalTest(unittest.TestCase):
-    """web.server's handling of conditional requests for cache validation."""
+    """
+    web.server's handling of conditional requests for cache validation.
+    """
 
     # XXX: test web.distrib.
 
@@ -353,17 +445,37 @@ class ConditionalTest(unittest.TestCase):
     def tearDown(self):
         self.channel.connectionLost(None)
 
-    def test_modified(self):
-        """If-Modified-Since cache validator (positive)"""
-        self.channel.lineReceived("If-Modified-Since: %s"
-                                  % http.datetimeToString(1))
+
+    def _modifiedTest(self, modifiedSince):
+        """
+        Given the value C{modifiedSince} for the I{If-Modified-Since}
+        header, verify that a response with a 200 code and the resource as
+        the body is returned.
+        """
+        self.channel.lineReceived("If-Modified-Since: " + modifiedSince)
         self.channel.lineReceived('')
         result = self.transport.getvalue()
         self.failUnlessEqual(httpCode(result), http.OK)
         self.failUnlessEqual(httpBody(result), "correct")
 
+
+    def test_modified(self):
+        """
+        If a request is made with an I{If-Modified-Since} header value with
+        a timestamp indicating a time before the last modification of the
+        requested resource, a 200 response is returned along with a response
+        body containing the resource.
+        """
+        self._modifiedTest(http.datetimeToString(1))
+
+
     def test_unmodified(self):
-        """If-Modified-Since cache validator (negative)"""
+        """
+        If a request is made with an I{If-Modified-Since} header value with
+        a timestamp indicating a time after the last modification of the
+        request resource, a 304 response is returned along with an empty
+        response body.
+        """
         self.channel.lineReceived("If-Modified-Since: %s"
                                   % http.datetimeToString(100))
         self.channel.lineReceived('')
@@ -371,6 +483,48 @@ class ConditionalTest(unittest.TestCase):
         self.failUnlessEqual(httpCode(result), http.NOT_MODIFIED)
         self.failUnlessEqual(httpBody(result), "")
 
+
+    def test_invalidTimestamp(self):
+        """
+        If a request is made with an I{If-Modified-Since} header value which
+        cannot be parsed, the header is treated as not having been present
+        and a normal 200 response is returned with a response body
+        containing the resource.
+        """
+        self._modifiedTest("like, maybe a week ago, I guess?")
+
+
+    def test_invalidTimestampYear(self):
+        """
+        If a request is made with an I{If-Modified-Since} header value which
+        contains a string in the year position which is not an integer, the
+        header is treated as not having been present and a normal 200
+        response is returned with a response body containing the resource.
+        """
+        self._modifiedTest("Thu, 01 Jan blah 00:00:10 GMT")
+
+
+    def test_invalidTimestampTooLongAgo(self):
+        """
+        If a request is made with an I{If-Modified-Since} header value which
+        contains a year before the epoch, the header is treated as not
+        having been present and a normal 200 response is returned with a
+        response body containing the resource.
+        """
+        self._modifiedTest("Thu, 01 Jan 1899 00:00:10 GMT")
+
+
+    def test_invalidTimestampMonth(self):
+        """
+        If a request is made with an I{If-Modified-Since} header value which
+        contains a string in the month position which is not a recognized
+        month abbreviation, the header is treated as not having been present
+        and a normal 200 response is returned with a response body
+        containing the resource.
+        """
+        self._modifiedTest("Thu, 01 Blah 1970 00:00:10 GMT")
+
+
     def test_etagMatchedNot(self):
         """If-None-Match ETag cache validator (positive)"""
         self.channel.lineReceived("If-None-Match: unmatchedTag")
@@ -388,6 +542,8 @@ class ConditionalTest(unittest.TestCase):
         self.failUnlessEqual(httpCode(result), http.NOT_MODIFIED)
         self.failUnlessEqual(httpBody(result), "")
 
+
+
 from twisted.web import google
 class GoogleTestCase(unittest.TestCase):
     def testCheckGoogle(self):
@@ -396,68 +552,22 @@ class GoogleTestCase(unittest.TestCase):
         d.addCallback(self.assertEquals, 'http://twistedmatrix.com/')
         return d
 
-from twisted.web import static
-from twisted.web import script
 
-class StaticFileTest(unittest.TestCase):
 
-    def testStaticPaths(self):
-        import os
-        dp = os.path.join(self.mktemp(),"hello")
-        ddp = os.path.join(dp, "goodbye")
-        tp = os.path.abspath(os.path.join(dp,"world.txt"))
-        tpy = os.path.join(dp,"wyrld.rpy")
-        os.makedirs(dp)
-        f = open(tp,"wb")
-        f.write("hello world")
-        f = open(tpy, "wb")
-        f.write("""
-from twisted.web.static import Data
-resource = Data('dynamic world','text/plain')
-""")
-        f = static.File(dp)
-        f.processors = {
-            '.rpy': script.ResourceScript,
-            }
 
-        f.indexNames = f.indexNames + ['world.txt']
-        self.assertEquals(f.getChild('', DummyRequest([''])).path,
-                          tp)
-        self.assertEquals(f.getChild('wyrld.rpy', DummyRequest(['wyrld.rpy'])
-                                     ).__class__,
-                          static.Data)
-        f = static.File(dp)
-        wtextr = DummyRequest(['world.txt'])
-        wtext = f.getChild('world.txt', wtextr)
-        self.assertEquals(wtext.path, tp)
-        wtext.render(wtextr)
-        self.assertEquals(wtextr.outgoingHeaders.get('content-length'),
-                          str(len('hello world')))
-        self.assertNotEquals(f.getChild('', DummyRequest([''])).__class__,
-                             static.File)
 
-    def testIgnoreExt(self):
-        f = static.File(".")
-        f.ignoreExt(".foo")
-        self.assertEquals(f.ignoredExts, [".foo"])
-        f = static.File(".")
-        self.assertEquals(f.ignoredExts, [])
-        f = static.File(".", ignoredExts=(".bar", ".baz"))
-        self.assertEquals(f.ignoredExts, [".bar", ".baz"])
+class RequestTests(unittest.TestCase):
+    """
+    Tests for the HTTP request class, L{server.Request}.
+    """
 
-    def testIgnoredExts(self):
-        import os
-        dp = os.path.join(self.mktemp(), 'allYourBase')
-        fp = os.path.join(dp, 'AreBelong.ToUs')
-        os.makedirs(dp)
-        open(fp, 'wb').write("Take off every 'Zig'!!")
-        f = static.File(dp)
-        f.ignoreExt('.ToUs')
-        dreq = DummyRequest([''])
-        child_without_ext = f.getChild('AreBelong', dreq)
-        self.assertNotEquals(child_without_ext, f.childNotFound)
+    def test_interface(self):
+        """
+        L{server.Request} instances provide L{iweb.IRequest}.
+        """
+        self.assertTrue(
+            verifyObject(iweb.IRequest, server.Request(DummyChannel(), True)))
 
-class TestRequest(unittest.TestCase):
 
     def testChildLink(self):
         request = server.Request(DummyChannel(), 1)
@@ -547,13 +657,6 @@ class TestRequest(unittest.TestCase):
         self.assertEqual(request.prePathURL(), 'http://example.com/foo%2Fbar')
 
 
-    def testNotifyFinishConnectionLost(self):
-        d = DummyChannel()
-        request = server.Request(d, 1)
-        finished = request.notifyFinish()
-        request.connectionLost(error.ConnectionDone("Connection done"))
-        return self.assertFailure(finished, error.ConnectionDone)
-
 
 class RootResource(resource.Resource):
     isLeaf=0
@@ -605,10 +708,42 @@ class NewRenderResource(resource.Resource):
         return "ho ho"
 
 
+
+class HeadlessResource(object):
+    """
+    A resource that implements GET but not HEAD.
+    """
+    implements(resource.IResource)
+
+    allowedMethods = ["GET"]
+
+    def render(self, request):
+        """
+        Leave the request open for future writes.
+        """
+        self.request = request
+        if request.method not in self.allowedMethods:
+            raise error.UnsupportedMethod(self.allowedMethods)
+        self.request.write("some data")
+        return server.NOT_DONE_YET
+
+
+
+
 class NewRenderTestCase(unittest.TestCase):
-    def _getReq(self):
+    """
+    Tests for L{server.Request.render}.
+    """
+    def _getReq(self, resource=None):
+        """
+        Create a request object with a stub channel and install the
+        passed resource at /newrender. If no resource is passed,
+        create one.
+        """
         d = DummyChannel()
-        d.site.resource.putChild('newrender', NewRenderResource())
+        if resource is None:
+            resource = NewRenderResource()
+        d.site.resource.putChild('newrender', resource)
         d.transport.port = 81
         request = server.Request(d, 1)
         request.setHost('example.com', 81)
@@ -640,13 +775,135 @@ class NewRenderTestCase(unittest.TestCase):
         self.assertEquals(-1, req.transport.getvalue().find('hi hi'))
 
 
-class SDResource(resource.Resource):
-    def __init__(self,default):  self.default=default
-    def getChildWithDefault(self,name,request):
-        d=defer.succeed(self.default)
-        return util.DeferredResource(d).getChildWithDefault(name, request)
+    def test_unsupportedHead(self):
+        """
+        HEAD requests against resource that only claim support for GET
+        should not include a body in the response.
+        """
+        resource = HeadlessResource()
+        req = self._getReq(resource)
+        req.requestReceived("HEAD", "/newrender", "HTTP/1.0")
+        headers, body = req.transport.getvalue().split('\r\n\r\n')
+        self.assertEqual(req.code, 200)
+        self.assertEqual(body, '')
 
-class SDTest(unittest.TestCase):
+
+
+class GettableResource(resource.Resource):
+    """
+    Used by AllowedMethodsTest to simulate an allowed method.
+    """
+    def render_GET(self):
+        pass
+
+    def render_fred_render_ethel(self):
+        """
+        The unusual method name is designed to test the culling method
+        in C{twisted.web.resource._computeAllowedMethods}.
+        """
+        pass
+
+
+
+class AllowedMethodsTest(unittest.TestCase):
+    """
+    'C{twisted.web.resource._computeAllowedMethods} is provided by a
+    default should the subclass not provide the method.
+    """
+
+
+    def _getReq(self):
+        """
+        Generate a dummy request for use by C{_computeAllowedMethod} tests.
+        """
+        d = DummyChannel()
+        d.site.resource.putChild('gettableresource', GettableResource())
+        d.transport.port = 81
+        request = server.Request(d, 1)
+        request.setHost('example.com', 81)
+        request.gotLength(0)
+        return request
+
+
+    def test_computeAllowedMethods(self):
+        """
+        C{_computeAllowedMethods} will search through the
+        'gettableresource' for all attributes/methods of the form
+        'render_{method}' ('render_GET', for example) and return a list of
+        the methods. 'HEAD' will always be included from the
+        resource.Resource superclass.
+        """
+        res = GettableResource()
+        allowedMethods = resource._computeAllowedMethods(res)
+        self.assertEquals(set(allowedMethods),
+                          set(['GET', 'HEAD', 'fred_render_ethel'])) 
+
+
+    def test_notAllowed(self):
+        """
+        When an unsupported method is requested, the default
+        L{_computeAllowedMethods} method will be called to determine the
+        allowed methods, and the HTTP 405 'Method Not Allowed' status will
+        be returned with the allowed methods will be returned in the
+        'Allow' header.
+        """
+        req = self._getReq()
+        req.requestReceived('POST', '/gettableresource', 'HTTP/1.0')
+        self.assertEquals(req.code, 405)
+        self.assertEquals(
+            set(req.responseHeaders.getRawHeaders('allow')[0].split(", ")),
+            set(['GET', 'HEAD','fred_render_ethel'])
+        )
+
+
+    def test_notAllowedQuoting(self):
+        """
+        When an unsupported method response is generated, an HTML message will
+        be displayed.  That message should include a quoted form of the URI and,
+        since that value come from a browser and shouldn't necessarily be
+        trusted.
+        """
+        req = self._getReq()
+        req.requestReceived('POST', '/gettableresource?'
+                            'value=<script>bad', 'HTTP/1.0')
+        self.assertEquals(req.code, 405)
+        renderedPage = req.transport.getvalue()
+        self.assertNotIn("<script>bad", renderedPage)
+        self.assertIn('&lt;script&gt;bad', renderedPage)
+
+
+    def test_notImplementedQuoting(self):
+        """
+        When an not-implemented method response is generated, an HTML message
+        will be displayed.  That message should include a quoted form of the
+        requested method, since that value come from a browser and shouldn't
+        necessarily be trusted.
+        """
+        req = self._getReq()
+        req.requestReceived('<style>bad', '/gettableresource', 'HTTP/1.0')
+        self.assertEquals(req.code, 501)
+        renderedPage = req.transport.getvalue()
+        self.assertNotIn("<style>bad", renderedPage)
+        self.assertIn('&lt;style&gt;bad', renderedPage)
+
+
+
+class SDResource(resource.Resource):
+    def __init__(self,default):
+        self.default = default
+
+
+    def getChildWithDefault(self, name, request):
+        d = defer.succeed(self.default)
+        resource = util.DeferredResource(d)
+        return resource.getChildWithDefault(name, request)
+
+
+
+class DeferredResourceTests(unittest.TestCase):
+    """
+    Tests for L{DeferredResource}.
+    """
 
     def testDeferredResource(self):
         r = resource.Resource()
@@ -657,6 +914,22 @@ class SDTest(unittest.TestCase):
         self.assertEqual(d.postpath, ['bar', 'baz'])
 
 
+    def test_render(self):
+        """
+        L{DeferredResource} uses the request object's C{render} method to
+        render the resource which is the result of the L{Deferred} being
+        handled.
+        """
+        rendered = []
+        request = DummyRequest([])
+        request.render = rendered.append
+
+        result = resource.Resource()
+        deferredResource = util.DeferredResource(defer.succeed(result))
+        deferredResource.render(request)
+        self.assertEquals(rendered, [result])
+
+
 
 class DummyRequestForLogTest(DummyRequest):
     uri = '/dummy' # parent class uri has "http://", which doesn't really happen
@@ -675,7 +948,7 @@ class TestLogEscaping(unittest.TestCase):
         self.request = DummyRequestForLogTest(self.site, False)
 
     def testSimple(self):
-        http._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
+        self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
         self.site.log(self.request)
         self.site.logFile.seek(0)
@@ -684,7 +957,7 @@ class TestLogEscaping(unittest.TestCase):
             '1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] "GET /dummy HTTP/1.0" 123 - "-" "-"\n')
 
     def testMethodQuote(self):
-        http._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
+        self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
         self.request.method = 'G"T'
         self.site.log(self.request)
@@ -694,7 +967,7 @@ class TestLogEscaping(unittest.TestCase):
             '1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] "G\\"T /dummy HTTP/1.0" 123 - "-" "-"\n')
 
     def testRequestQuote(self):
-        http._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
+        self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
         self.request.uri='/dummy"withquote'
         self.site.log(self.request)
@@ -704,7 +977,7 @@ class TestLogEscaping(unittest.TestCase):
             '1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] "GET /dummy\\"withquote HTTP/1.0" 123 - "-" "-"\n')
 
     def testProtoQuote(self):
-        http._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
+        self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
         self.request.clientproto='HT"P/1.0'
         self.site.log(self.request)
@@ -714,7 +987,7 @@ class TestLogEscaping(unittest.TestCase):
             '1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] "GET /dummy HT\\"P/1.0" 123 - "-" "-"\n')
 
     def testRefererQuote(self):
-        http._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
+        self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
         self.request.headers['referer'] = 'http://malicious" ".website.invalid'
         self.site.log(self.request)
@@ -724,7 +997,7 @@ class TestLogEscaping(unittest.TestCase):
             '1.2.3.4 - - [25/Oct/2004:12:31:59 +0000] "GET /dummy HTTP/1.0" 123 - "http://malicious\\" \\".website.invalid" "-"\n')
 
     def testUserAgentQuote(self):
-        http._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
+        self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
         self.request.headers['user-agent'] = 'Malicious Web" Evil'
         self.site.log(self.request)
