@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.web.test.test_xmlrpc -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -9,8 +9,8 @@ Maintainer: Itamar Shtull-Trauring
 """
 
 # System Imports
-import xmlrpclib
-import urlparse
+import sys, xmlrpclib, urlparse
+
 
 # Sibling Imports
 from twisted.web import resource, server, http
@@ -28,6 +28,32 @@ Binary = xmlrpclib.Binary
 Boolean = xmlrpclib.Boolean
 DateTime = xmlrpclib.DateTime
 
+# On Python 2.4 and earlier, DateTime.decode returns unicode.
+if sys.version_info[:2] < (2, 5):
+    _decode = DateTime.decode
+    DateTime.decode = lambda self, value: _decode(self, value.encode('ascii'))
+
+
+def withRequest(f):
+    """
+    Decorator to cause the request to be passed as the first argument
+    to the method.
+
+    If an I{xmlrpc_} method is wrapped with C{withRequest}, the
+    request object is passed as the first argument to that method.
+    For example::
+
+        @withRequest
+        def xmlrpc_echo(self, request, s):
+            return s
+
+    @since: 10.2
+    """
+    f.withRequest = True
+    return f
+
+
+
 class NoSuchFunction(Fault):
     """
     There is no function by the given name.
@@ -76,6 +102,13 @@ class XMLRPC(resource.Resource):
     Sub-handlers for prefixed methods (e.g., system.listMethods)
     can be added with putSubHandler. By default, prefixes are
     separated with a '.'. Override self.separator to change this.
+
+    @ivar allowNone: Permit XML translating of Python constant None.
+    @type allowNone: C{bool}
+
+    @ivar useDateTime: Present datetime values as datetime.datetime objects?
+        Requires Python >= 2.5.
+    @type useDateTime: C{bool}
     """
 
     # Error codes for Twisted, if they conflict with yours then
@@ -87,10 +120,18 @@ class XMLRPC(resource.Resource):
     separator = '.'
     allowedMethods = ('POST',)
 
-    def __init__(self, allowNone=False):
+    def __init__(self, allowNone=False, useDateTime=False):
         resource.Resource.__init__(self)
         self.subHandlers = {}
         self.allowNone = allowNone
+        self.useDateTime = useDateTime
+
+
+    def __setattr__(self, name, value):
+        if name == "useDateTime" and value and sys.version_info[:2] < (2, 5):
+            raise RuntimeError("useDateTime requires Python 2.5 or later.")
+        self.__dict__[name] = value
+
 
     def putSubHandler(self, prefix, handler):
         self.subHandlers[prefix] = handler
@@ -105,7 +146,12 @@ class XMLRPC(resource.Resource):
         request.content.seek(0, 0)
         request.setHeader("content-type", "text/xml")
         try:
-            args, functionPath = xmlrpclib.loads(request.content.read())
+            if self.useDateTime:
+                args, functionPath = xmlrpclib.loads(request.content.read(),
+                    use_datetime=True)
+            else:
+                # Maintain backwards compatibility with Python < 2.5
+                args, functionPath = xmlrpclib.loads(request.content.read())
         except Exception, e:
             f = Fault(self.FAILURE, "Can't deserialize input: %s" % (e,))
             self._cbRender(f, request)
@@ -115,29 +161,45 @@ class XMLRPC(resource.Resource):
             except Fault, f:
                 self._cbRender(f, request)
             else:
-                defer.maybeDeferred(function, *args).addErrback(
-                    self._ebRender
-                ).addCallback(
-                    self._cbRender, request
-                )
+                # Use this list to track whether the response has failed or not.
+                # This will be used later on to decide if the result of the
+                # Deferred should be written out and Request.finish called.
+                responseFailed = []
+                request.notifyFinish().addErrback(responseFailed.append)
+                if getattr(function, 'withRequest', False):
+                    d = defer.maybeDeferred(function, request, *args)
+                else:
+                    d = defer.maybeDeferred(function, *args)
+                d.addErrback(self._ebRender)
+                d.addCallback(self._cbRender, request, responseFailed)
         return server.NOT_DONE_YET
 
-    def _cbRender(self, result, request):
+
+    def _cbRender(self, result, request, responseFailed=None):
+        if responseFailed:
+            return
+
         if isinstance(result, Handler):
             result = result.result
         if not isinstance(result, Fault):
             result = (result,)
         try:
-            s = xmlrpclib.dumps(result, methodresponse=True,
-                                allow_none=self.allowNone)
-        except Exception, e:
-            f = Fault(self.FAILURE, "Can't serialize output: %s" % (e,))
-            s = xmlrpclib.dumps(f, methodresponse=True,
-                                allow_none=self.allowNone)
-        request.setHeader("content-length", str(len(s)))
-        request.write(s)
+            try:
+                content = xmlrpclib.dumps(
+                    result, methodresponse=True,
+                    allow_none=self.allowNone)
+            except Exception, e:
+                f = Fault(self.FAILURE, "Can't serialize output: %s" % (e,))
+                content = xmlrpclib.dumps(f, methodresponse=True,
+                                          allow_none=self.allowNone)
+
+            request.setHeader("content-length", str(len(content)))
+            request.write(content)
+        except:
+            log.err()
         request.finish()
 
+
     def _ebRender(self, failure):
         if isinstance(failure.value, Fault):
             return failure.value
@@ -200,8 +262,8 @@ class XMLRPCIntrospection(XMLRPC):
         Implement Introspection support for an XMLRPC server.
 
         @param parent: the XMLRPC server to add Introspection support to.
+        @type parent: L{XMLRPC}
         """
-
         XMLRPC.__init__(self)
         self._xmlrpc_parent = parent
 
@@ -251,7 +313,8 @@ def addIntrospection(xmlrpc):
     """
     Add Introspection support to an XMLRPC server.
 
-    @param xmlrpc: The xmlrpc server to add Introspection support to.
+    @param parent: the XMLRPC server to add Introspection support to.
+    @type parent: L{XMLRPC}
     """
     xmlrpc.putSubHandler('system', XMLRPCIntrospection(xmlrpc))
 
@@ -259,6 +322,7 @@ def addIntrospection(xmlrpc):
 class QueryProtocol(http.HTTPClient):
 
     def connectionMade(self):
+        self._response = None
         self.sendCommand('POST', self.factory.path)
         self.sendHeader('User-Agent', 'Twisted/XMLRPClib')
         self.sendHeader('Host', self.factory.host)
@@ -276,7 +340,27 @@ class QueryProtocol(http.HTTPClient):
             self.factory.badStatus(status, message)
 
     def handleResponse(self, contents):
-        self.factory.parseResponse(contents)
+        """
+        Handle the XML-RPC response received from the server.
+
+        Specifically, disconnect from the server and store the XML-RPC
+        response so that it can be properly handled when the disconnect is
+        finished.
+        """
+        self.transport.loseConnection()
+        self._response = contents
+
+    def connectionLost(self, reason):
+        """
+        The connection to the server has been lost.
+
+        If we have a full response from the server, then parse it and fired a
+        Deferred with the return value or C{Fault} that the server gave us.
+        """
+        http.HTTPClient.connectionLost(self, reason)
+        if self._response is not None:
+            response, self._response = self._response, None
+            self.factory.parseResponse(response)
 
 
 payloadTemplate = """<?xml version="1.0"?>
@@ -288,23 +372,66 @@ payloadTemplate = """<?xml version="1.0"?>
 
 
 class _QueryFactory(protocol.ClientFactory):
+    """
+    XML-RPC Client Factory
+
+    @ivar path: The path portion of the URL to which to post method calls.
+    @type path: C{str}
+
+    @ivar host: The value to use for the Host HTTP header.
+    @type host: C{str}
+
+    @ivar user: The username with which to authenticate with the server
+        when making calls.
+    @type user: C{str} or C{NoneType}
+
+    @ivar password: The password with which to authenticate with the server
+        when making calls.
+    @type password: C{str} or C{NoneType}
+
+    @ivar useDateTime: Accept datetime values as datetime.datetime objects.
+        also passed to the underlying xmlrpclib implementation.  Default to
+        False.  Requires Python >= 2.5.
+    @type useDateTime: C{bool}
+    """
 
     deferred = None
     protocol = QueryProtocol
 
     def __init__(self, path, host, method, user=None, password=None,
-                 allowNone=False, args=()):
+                 allowNone=False, args=(), canceller=None, useDateTime=False):
+        """
+        @param method: The name of the method to call.
+        @type method: C{str}
+
+        @param allowNone: allow the use of None values in parameters. It's
+            passed to the underlying xmlrpclib implementation. Default to False.
+        @type allowNone: C{bool} or C{NoneType}
+
+        @param args: the arguments to pass to the method.
+        @type args: C{tuple}
+
+        @param canceller: A 1-argument callable passed to the deferred as the
+            canceller callback.
+        @type canceller: callable or C{NoneType}
+        """
         self.path, self.host = path, host
         self.user, self.password = user, password
         self.payload = payloadTemplate % (method,
             xmlrpclib.dumps(args, allow_none=allowNone))
-        self.deferred = defer.Deferred()
+        self.deferred = defer.Deferred(canceller)
+        self.useDateTime = useDateTime
 
     def parseResponse(self, contents):
         if not self.deferred:
             return
         try:
-            response = xmlrpclib.loads(contents)[0][0]
+            if self.useDateTime:
+                response = xmlrpclib.loads(contents,
+                    use_datetime=True)[0][0]
+            else:
+                # Maintain backwards compatibility with Python < 2.5
+                response = xmlrpclib.loads(contents)[0][0]
         except:
             deferred, self.deferred = self.deferred, None
             deferred.errback(failure.Failure())
@@ -334,34 +461,48 @@ class Proxy:
     Use proxy.callRemote('foobar', *args) to call remote method
     'foobar' with *args.
 
+    @ivar user: The username with which to authenticate with the server
+        when making calls.  If specified, overrides any username information
+        embedded in C{url}.  If not specified, a value may be taken from
+        C{url} if present.
+    @type user: C{str} or C{NoneType}
+
+    @ivar password: The password with which to authenticate with the server
+        when making calls.  If specified, overrides any password information
+        embedded in C{url}.  If not specified, a value may be taken from
+        C{url} if present.
+    @type password: C{str} or C{NoneType}
+
+    @ivar allowNone: allow the use of None values in parameters. It's
+        passed to the underlying xmlrpclib implementation. Default to False.
+    @type allowNone: C{bool} or C{NoneType}
+
+    @ivar useDateTime: Accept datetime values as datetime.datetime objects.
+        also passed to the underlying xmlrpclib implementation.  Default to
+        False.  Requires Python >= 2.5.
+    @type useDateTime: C{bool}
+
+    @ivar connectTimeout: Number of seconds to wait before assuming the
+        connection has failed.
+    @type connectTimeout: C{float}
+
+    @ivar _reactor: the reactor used to create connections.
+    @type _reactor: object providing L{twisted.internet.interfaces.IReactorTCP}
+
     @ivar queryFactory: object returning a factory for XML-RPC protocol. Mainly
         useful for tests.
     """
     queryFactory = _QueryFactory
 
-    def __init__(self, url, user=None, password=None, allowNone=False):
+    def __init__(self, url, user=None, password=None, allowNone=False,
+                 useDateTime=False, connectTimeout=30.0, reactor=reactor):
         """
-        @type url: C{str}
         @param url: The URL to which to post method calls.  Calls will be made
-        over SSL if the scheme is HTTPS.  If netloc contains username or
-        password information, these will be used to authenticate, as long as
-        the C{user} and C{password} arguments are not specified.
+            over SSL if the scheme is HTTPS.  If netloc contains username or
+            password information, these will be used to authenticate, as long as
+            the C{user} and C{password} arguments are not specified.
+        @type url: C{str}
 
-        @type user: C{str} or None
-        @param user: The username with which to authenticate with the server
-        when making calls.  If specified, overrides any username information
-        embedded in C{url}.  If not specified, a value may be taken from C{url}
-        if present.
-
-        @type password: C{str} or None
-        @param password: The password with which to authenticate with the
-        server when making calls.  If specified, overrides any password
-        information embedded in C{url}.  If not specified, a value may be taken
-        from C{url} if present.
-
-        @type allowNone: C{bool} or None
-        @param allowNone: allow the use of None values in parameters. It's
-        passed to the underlying xmlrpclib implementation. Default to False.
         """
         scheme, netloc, path, params, query, fragment = urlparse.urlparse(url)
         netlocParts = netloc.split('@')
@@ -389,6 +530,16 @@ class Proxy:
         if password is not None:
             self.password = password
         self.allowNone = allowNone
+        self.useDateTime = useDateTime
+        self.connectTimeout = connectTimeout
+        self._reactor = reactor
+
+
+    def __setattr__(self, name, value):
+        if name == "useDateTime" and value and sys.version_info[:2] < (2, 5):
+            raise RuntimeError("useDateTime requires Python 2.5 or later.")
+        self.__dict__[name] = value
+
 
     def callRemote(self, method, *args):
         """
@@ -398,16 +549,28 @@ class Proxy:
             or a failure if the method failed. Generally, the failure type will
             be L{Fault}, but you can also have an C{IndexError} on some buggy
             servers giving empty responses.
+
+            If the deferred is cancelled before the request completes, the
+            connection is closed and the deferred will fire with a
+            L{defer.CancelledError}.
         """
+        def cancel(d):
+            factory.deferred = None
+            connector.disconnect()
         factory = self.queryFactory(
             self.path, self.host, method, self.user,
-            self.password, self.allowNone, args)
+            self.password, self.allowNone, args, cancel, self.useDateTime)
+
         if self.secure:
             from twisted.internet import ssl
-            reactor.connectSSL(self.host, self.port or 443,
-                               factory, ssl.ClientContextFactory())
+            connector = self._reactor.connectSSL(
+                self.host, self.port or 443,
+                factory, ssl.ClientContextFactory(),
+                timeout=self.connectTimeout)
         else:
-            reactor.connectTCP(self.host, self.port or 80, factory)
+            connector = self._reactor.connectTCP(
+                self.host, self.port or 80, factory,
+                timeout=self.connectTimeout)
         return factory.deferred
 
 
